@@ -1,5 +1,5 @@
 //
-// GOATTRACKER v2.74
+// GOATTRACKER v2.75
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -65,6 +65,10 @@ unsigned residdelay = 0;
 unsigned hardsidbufinteractive = 20;
 unsigned hardsidbufplayback = 400;
 float basepitch = 0.0f;
+float equaldivisionsperoctave = 12.0f;
+int tuningcount = 0;
+double tuning[96];
+extern unsigned bigwindow;
 
 char configbuf[MAX_PATHNAME];
 char loadedsongfilename[MAX_FILENAME];
@@ -76,7 +80,11 @@ char instrfilter[MAX_FILENAME];
 char instrpath[MAX_PATHNAME];
 char packedpath[MAX_PATHNAME];
 
-char *programname = "$VER: GoatTracker v2.74";
+extern char *notename[];
+char *programname = "$VER: GoatTracker v2.75";
+char specialnotenames[186];
+char scalatuningfilepath[MAX_PATHNAME];
+char tuningname[64];
 
 char textbuffer[MAX_PATHNAME];
 
@@ -97,17 +105,21 @@ char* usage[] = {
     "-Gxx Set pitch of A-4 in Hz (0 = use default frequencytable, close to 440Hz)",
     "-Hxx Use HardSID (0 = off, 1 = HardSID ID0 2 = HardSID ID1 etc.)",
     "-Ixx Set reSID interpolation (0 = off, 1 = on, 2 = distortion, 3 = distortion & on) DEFAULT=off",
-    "-Kxx Note-entry mode (0 = PROTRACKER 1 = DMC) DEFAULT=PROTRK.",
+	  "-Jxx Set special note names (2 chars for every note in an octave/cycle, e.g. C-DbD-EbE-F-GbG-AbA-BbB-)",
+    "-Kxx Note-entry mode (0 = Protracker, 1 = DMC, 2 = Janko) DEFAULT=Protracker",
     "-Lxx SID memory location in hex. DEFAULT=D400",
     "-Mxx Set sound mixing rate DEFAULT=44100",
     "-Oxx Set pulseoptimization/skipping (0 = off, 1 = on) DEFAULT=on",
+    "-Qxx Set equal divisions per octave (12 = default, 8.2019143 = Bohlen-Pierce)",
     "-Rxx Set realtime-effect optimization/skipping (0 = off, 1 = on) DEFAULT=on",
     "-Sxx Set speed multiplier (0 for 25Hz, 1 for 1x, 2 for 2x etc.)",
     "-Txx Set HardSID interactive mode sound buffer length in milliseconds DEFAULT=20, max.buffering=0",
     "-Uxx Set HardSID playback mode sound buffer length in milliseconds DEFAULT=400, max.buffering=0",
     "-Vxx Set finevibrato conversion (0 = off, 1 = on) DEFAULT=on",
     "-Xxx Set window type (0 = window, 1 = fullscreen) DEFAULT=window",
+    "-Yxx Path to a Scala tuning file .scl",
     "-Zxx Set random reSID write delay in cycles (0 = off) DEFAULT=off",
+    "-wxx Set window scale factor (1 = no scaling, 2 to 4 = 2 to 4 times bigger window) DEFAULT=1",
     "-N   Use NTSC timing",
     "-P   Use PAL timing (DEFAULT)",
     "-W   Write sound output to a file SIDAUDIO.RAW",
@@ -177,7 +189,11 @@ int main(int argc, char **argv)
     getfloatparam(configfile, &filterparams.type4b);
     getfloatparam(configfile, &filterparams.voicenonlinearity);
     getparam(configfile, (unsigned*)&win_fullscreen);
+    getparam(configfile, &bigwindow);
     getfloatparam(configfile, &basepitch);
+    getfloatparam(configfile, &equaldivisionsperoctave);
+    getstringparam(configfile, specialnotenames);
+    getstringparam(configfile, scalatuningfilepath);
     fclose(configfile);
   }
 
@@ -194,7 +210,7 @@ int main(int argc, char **argv)
     #endif
     {
       int y;
-      switch(toupper(argv[c][1]))
+      switch (argv[c][1]) //switch (toupper(argv[c][1]))
       {
         case '-':
         if (strcmp(argv[c], "--help"))
@@ -312,6 +328,22 @@ int main(int argc, char **argv)
         case 'G':
         sscanf(&argv[c][2], "%f", &basepitch);
         break;
+ 
+        case 'Q':
+        sscanf(&argv[c][2], "%f", &equaldivisionsperoctave);
+        break;
+ 
+        case 'J':
+        sscanf(&argv[c][2], "%s", specialnotenames);
+        break;
+  
+        case 'Y':
+        sscanf(&argv[c][2], "%s", scalatuningfilepath);
+        break;
+  
+        case 'w':
+        sscanf(&argv[c][2], "%u", &bigwindow);
+        break;
       }
     }
     else
@@ -350,12 +382,24 @@ int main(int argc, char **argv)
   if (residdelay > 63) residdelay = 63;
   if (customclockrate < 100) customclockrate = 0;
 
+  // Read Scala tuning file
+  if (scalatuningfilepath[0] != '0' && scalatuningfilepath[1] != '\0')
+  {
+    readscalatuningfile();
+  }
+
   // Calculate frequencytable if necessary
   if (basepitch < 0.0f)
     basepitch = 0.0f;
   if (basepitch > 0.0f)
     calculatefreqtable();
 
+  // Set special note names
+  if (specialnotenames[1] != '\0')
+  {
+    setspecialnotenames();
+  }
+
   // Set screenmode
   if (!initscreen())
     return EXIT_FAILURE;
@@ -413,7 +457,7 @@ int main(int argc, char **argv)
                         ";Packer/relocator player address\n$%04x\n\n"
                         ";Packer/relocator zeropage baseaddress\n$%02x\n\n"
                         ";Packer/relocator player type (0 = standard ... 3 = minimal)\n%d\n\n"
-                        ";Key entry mode (0 = Protracker, 1 = DMC)\n%d\n\n"
+                        ";Key entry mode (0 = Protracker, 1 = DMC, 2 = Janko)\n%d\n\n"
                         ";Pattern highlight step size\n%d\n\n"
                         ";Speed multiplier (0 = 25Hz, 1 = 1X, 2 = 2X etc.)\n%d\n\n"
                         ";Use CatWeasel SID (0 = off, 1 = on)\n%d\n\n"
@@ -439,7 +483,11 @@ int main(int argc, char **argv)
                         ";reSID-fp type 4 b\n%f\n\n"
                         ";reSID-fp voice nonlinearity\n%f\n\n"
                         ";Window type (0 = window, 1 = fullscreen)\n%d\n\n"
-                        ";Base pitch of A-4 in Hz (0 = use default frequencytable)\n%f\n\n",
+				            		";window scale factor (1 = no scaling, 2 to 4 = 2 to 4 times bigger window)\n%d\n\n"
+                        ";Base pitch of A-4 in Hz (0 = use default frequencytable)\n%f\n\n"
+                        ";Equal divisions per octave (12 = default, 8.2019143 = Bohlen-Pierce)\n%f\n\n"
+			            			";Special note names (2 chars for every note in an octave/cycle)\n%s\n\n"
+				            		";Path to a Scala tuning file .scl\n%s\n\n",				
     b,
     mr,
     hardsid,
@@ -475,7 +523,11 @@ int main(int argc, char **argv)
     filterparams.type4b,
     filterparams.voicenonlinearity,
     win_fullscreen,
-    basepitch);
+    bigwindow,
+    basepitch,
+	  equaldivisionsperoctave,
+	  specialnotenames,
+	  scalatuningfilepath);
     fclose(configfile);
   }
 
@@ -1386,6 +1438,22 @@ void getfloatparam(FILE *handle, float *value)
   sscanf(configptr, "%f", value);
 }
 
+void getstringparam(FILE *handle, char *value)
+{
+  char *configptr;
+
+  for (;;)
+  {
+    if (feof(handle)) return;
+    fgets(configbuf, MAX_PATHNAME, handle);
+    if ((configbuf[0]) && (configbuf[0] != ';') && (configbuf[0] != ' ') && (configbuf[0] != 13) && (configbuf[0] != 10)) break;
+  }
+
+  configptr = configbuf;
+  
+  sscanf(configptr, "%s", value);
+}
+
 void prevmultiplier(void)
 {
   if (multiplier > 0)
@@ -1407,16 +1475,156 @@ void nextmultiplier(void)
 void calculatefreqtable()
 {
   double basefreq = (double)basepitch * (16777216.0 / 985248.0) * pow(2.0, 0.25) / 32.0;
+  double cyclebasefreq = basefreq;
+  double freq = basefreq;
   int c;
+  int i;
 
-  for (c = 0; c < 8*12 ; c++)
+  if (tuningcount)
   {
-    double note = c;
-    double freq = basefreq * pow(2.0, note/12.0);
-    int intfreq = freq + 0.5;
-    if (intfreq > 0xffff)
-        intfreq = 0xffff;
-    freqtbllo[c] = intfreq & 0xff;
-    freqtblhi[c] = intfreq >> 8;
+    c = 0;
+    while (c < 96)
+    {
+      for (i = 0; i < tuningcount; i++)
+      {
+        if (c < 96)
+        {
+          int intfreq = freq + 0.5;
+          if (intfreq > 0xffff)
+              intfreq = 0xffff;
+          freqtbllo[c] = intfreq & 0xff;
+          freqtblhi[c] = intfreq >> 8;          
+          freq = cyclebasefreq * tuning[i];
+          c++;
+        }
+      }
+      cyclebasefreq = freq;
+    }
+  }
+  else
+  {
+    for (c = 0; c < 8*12 ; c++)
+    {
+      double note = c;
+      double freq = basefreq * pow(2.0, note/(double)equaldivisionsperoctave);
+      int intfreq = freq + 0.5;
+      if (intfreq > 0xffff)
+          intfreq = 0xffff;
+      freqtbllo[c] = intfreq & 0xff;
+      freqtblhi[c] = intfreq >> 8;
+    }
   }
 }
+
+void setspecialnotenames()
+{
+  int i;
+  int j;
+  int oct;
+  char *name;
+  char octave[11];
+  
+  i = 0;
+  oct = 0;
+  while (i < 93)
+  {
+    for (j = 0; j < 186; j += 2)
+    {
+      if (specialnotenames[j] == '\0')
+        break;
+	  if (i < 93)
+      {
+        name = malloc(4);
+        strncpy(name, specialnotenames + j, 2);
+        sprintf(octave, "%d", oct);
+        strcpy(name + 2, octave);
+        notename[i] = name;
+        i++;
+      }
+    }
+	oct++;
+  }
+}
+
+void readscalatuningfile()
+{
+  FILE *scalatuningfile;
+  char *configptr;
+  char strbuf[64];
+  char name[3];
+  int i;
+  double numerator;
+  double denominator;
+  double centvalue;
+	
+  scalatuningfile = fopen(scalatuningfilepath, "rt");
+  if (scalatuningfile)
+  {
+    // Tuning name
+    for (;;)
+    {
+      if (feof(scalatuningfile)) return;
+      fgets(configbuf, MAX_PATHNAME, scalatuningfile);
+      if ((configbuf[0]) && (configbuf[0] != '!') && (configbuf[0] != 13) && (configbuf[0] != 10)) break;
+    }
+    configptr = configbuf;
+    sscanf(configptr, "%63[^\t\n]", tuningname);
+	
+    // Tuning count
+    for (;;)
+    {
+      if (feof(scalatuningfile)) return;
+      fgets(configbuf, MAX_PATHNAME, scalatuningfile);
+      if ((configbuf[0]) && (configbuf[0] != '!') && (configbuf[0] != 13) && (configbuf[0] != 10)) break;
+    }
+    configptr = configbuf;
+    sscanf(configptr, "%d", &tuningcount);
+	
+    // Tunings 
+	  for (i = 0; i < tuningcount; i++)
+    {
+      for (;;)
+      {
+        if (feof(scalatuningfile)) return;
+        fgets(configbuf, MAX_PATHNAME, scalatuningfile);
+        if ((configbuf[0]) && (configbuf[0] != '!') && (configbuf[0] != 13) && (configbuf[0] != 10)) break;
+      }
+      configptr = configbuf;
+      name[0] = '\0';
+      sscanf(configptr, "%63s %2s", strbuf, name);
+      if (!i)
+      {
+        strcpy(specialnotenames, name);
+      }
+      else
+      {
+        if (i == tuningcount - 1)
+        {
+          char *tmp = strdup(specialnotenames);
+          strcpy(specialnotenames, name);
+          strcat(specialnotenames, tmp);
+          free(tmp);
+        }
+        else
+        {
+          strcat(specialnotenames, name);
+        }
+      }
+      if (!strchr(strbuf, '.'))
+      {
+        sscanf(strbuf, "%lf", &numerator);
+        if (strchr(strbuf, '/'))
+        {
+          sscanf(strchr(strbuf, '/') + 1, "%lf", &denominator);
+          tuning[i] = numerator / denominator;
+        }
+      }
+      else
+      {
+        sscanf(configptr, "%lf", &centvalue);
+        tuning[i] = pow(2.0, centvalue / 1200.0);
+      }
+    }
+    fclose(scalatuningfile);
+  }  
+}
